@@ -29,19 +29,29 @@ using System.Collections;
 using System.IO;
 using System.Threading;
 
-using Mono.Data.SqliteClient;
+using Mono.Data.Sqlite;
 
 using Beagle.Util;
 
 namespace Beagle.Daemon {
 
+	// FIXME: This class isn't multithread safe!  This class does not
+	// ensure that different threads don't utilize a transaction started
+	// in a certain thread at the same time.  However, since all the
+	// writes to this database are handled by a single thread, this isn't
+	// currently a problem, but it would have to be fixed if we wanted to
+	// do this in the future.
+	//
+	// See http://bugzilla.gnome.org/show_bug.cgi?329022
+
 	public class FileAttributesStore_Sqlite : IFileAttributesStore {
 
 		// Version history:
 		// 1: Original version
 		// 2: Replaced LastIndexedTime with LastAttrTime
 		// 3: Changed STRING to TEXT
-		const int VERSION = 3;
+		// 4: Use (directory, filename) as unique constraint
+		const int VERSION = 4;
 
 		private SqliteConnection connection;
 		private BitArray path_flags;
@@ -125,24 +135,20 @@ namespace Beagle.Daemon {
 							")");
 
 				SqliteUtils.DoNonQuery (connection,
-							"INSERT INTO db_info (version, fingerprint) VALUES ({0}, '{1}')",
-							VERSION, index_fingerprint);
+							"INSERT INTO db_info (version, fingerprint) VALUES (@version, @index_fingerprint)",
+							new string [] {"@version", "@index_fingerprint"},
+							new object [] {VERSION, index_fingerprint});
 
 				SqliteUtils.DoNonQuery (connection,
-							"CREATE TABLE file_attributes (           " +
-							"  unique_id      TEXT UNIQUE,          " +
+							"CREATE TABLE file_attributes (         " +
+							"  unique_id      TEXT NOT NULL,        " +
 							"  directory      TEXT NOT NULL,        " +
 							"  filename       TEXT NOT NULL,        " +
 							"  last_mtime     TEXT NOT NULL,        " +
 							"  last_attrtime  TEXT NOT NULL,        " +
 							"  filter_name    TEXT NOT NULL,        " +
-							"  filter_version TEXT NOT NULL         " +
-							")");
-
-				SqliteUtils.DoNonQuery (connection,
-							"CREATE UNIQUE INDEX file_path on file_attributes (" +
-							"  directory,      " +
-							"  filename        " +
+							"  filter_version TEXT NOT NULL,        " +
+							"  UNIQUE (directory, filename)         " +
 							")");
 			} else {
 				SqliteCommand command;
@@ -188,8 +194,7 @@ namespace Beagle.Daemon {
 		{
 			SqliteConnection c;			
 			c = new SqliteConnection ();
-			c.ConnectionString = "version=" + ExternalStringsHack.SqliteVersion
-				+ ",encoding=UTF-8,URI=file:" + GetDbPath (directory);
+			c.ConnectionString = "version=3,encoding=UTF-8,URI=file:" + GetDbPath (directory);
 			c.Open ();
 			return c;
 		}
@@ -291,13 +296,15 @@ namespace Beagle.Daemon {
 				ret = SqliteUtils.DoNonQuery (connection,
 							"INSERT OR REPLACE INTO file_attributes " +
 							" (unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version) " +
-							" VALUES ('{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}')",
-							GuidFu.ToShortString (fa.UniqueId),
-							fa.Directory.Replace ("'", "''"), fa.Filename.Replace ("'", "''"),
-							StringFu.DateTimeToString (fa.LastWriteTime),
-							StringFu.DateTimeToString (fa.LastAttrTime),
-							filter_name,
-							fa.FilterVersion);
+							" VALUES (@unique_id, @directory, @filename, @last_mtime, @last_attrtime, @filter_name, @filter_version)",
+							new string [] {"@unique_id", "@directory", "@filename", "@last_mtime", "@last_attrtime", "@filter_name", "@filter_version"},
+							new object [] {
+								GuidFu.ToShortString (fa.UniqueId),
+								fa.Directory.Replace ("'", "''"), fa.Filename.Replace ("'", "''"),
+								StringFu.DateTimeToString (fa.LastWriteTime),
+								StringFu.DateTimeToString (fa.LastAttrTime),
+								filter_name,
+								fa.FilterVersion});
 			}
 
 			return (ret != 0);
@@ -322,8 +329,9 @@ namespace Beagle.Daemon {
 				MaybeStartTransaction ();
 
 				SqliteUtils.DoNonQuery (connection,
-							"DELETE FROM file_attributes WHERE directory='{0}' AND filename='{1}'",
-							directory, filename);
+							"DELETE FROM file_attributes WHERE directory=@directory AND filename=@filename",
+							new string [] {"@directory", "@filename"},
+							new object [] {directory, filename});
 			}
 		}
 
