@@ -13,15 +13,18 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 using System;
 using Directory = Lucene.Net.Store.Directory;
 using IndexInput = Lucene.Net.Store.IndexInput;
 using IndexOutput = Lucene.Net.Store.IndexOutput;
+using Constants = Lucene.Net.Util.Constants;
+
 namespace Lucene.Net.Index
 {
-
+	
 	[Serializable]
-	sealed public class SegmentInfos : System.Collections.ArrayList
+	public sealed class SegmentInfos : System.Collections.ArrayList
 	{
 		
 		/// <summary>The file format version, a negative number. </summary>
@@ -29,7 +32,10 @@ namespace Lucene.Net.Index
 		public const int FORMAT = - 1;
 		
 		public int counter = 0; // used to name new segments
-		private long version = 0; //counts how often the index has been changed by adding or deleting docs
+		/// <summary> counts how often the index has been changed by adding or deleting docs.
+		/// starting with the current time in milliseconds forces to create unique version numbers.
+		/// </summary>
+		private long version = System.DateTime.Now.Ticks;
 		
 		public SegmentInfo Info(int i)
 		{
@@ -39,7 +45,7 @@ namespace Lucene.Net.Index
 		public void  Read(Directory directory)
 		{
 			
-			IndexInput input = directory.OpenInput("segments");
+			IndexInput input = directory.OpenInput(IndexFileNames.SEGMENTS);
 			try
 			{
 				int format = input.ReadInt();
@@ -69,7 +75,7 @@ namespace Lucene.Net.Index
 				{
 					// in old format the version number may be at the end of the file
 					if (input.GetFilePointer() >= input.Length())
-						version = 0;
+						version = (System.DateTime.Now.Ticks - 621355968000000000) / 10000;
 					// old file format without version number
 					else
 						version = input.ReadLong(); // read version
@@ -103,7 +109,7 @@ namespace Lucene.Net.Index
 			}
 			
 			// install new segment info
-			directory.RenameFile("segments.new", "segments");
+			directory.RenameFile("segments.new", IndexFileNames.SEGMENTS);
 		}
 		
 		/// <summary> version number when this SegmentInfos was generated.</summary>
@@ -116,7 +122,7 @@ namespace Lucene.Net.Index
 		public static long ReadCurrentVersion(Directory directory)
 		{
 			
-			IndexInput input = directory.OpenInput("segments");
+			IndexInput input = directory.OpenInput(IndexFileNames.SEGMENTS);
 			int format = 0;
 			long version = 0;
 			try
@@ -144,5 +150,26 @@ namespace Lucene.Net.Index
 			sis.Read(directory);
 			return sis.GetVersion();
 		}
+
+		public void Optimize(Directory directory)
+		{
+			string[] files = directory.List();
+
+			System.Collections.ArrayList segment_names = new System.Collections.ArrayList();
+			foreach (SegmentInfo si in this)
+				segment_names.Add (si.name);
+
+			foreach (string file in files) {
+				string basename = System.IO.Path.GetFileNameWithoutExtension (file);
+				if (segment_names.Contains (basename))
+					continue;
+
+				if (basename == IndexFileNames.DELETABLE || basename == IndexFileNames.SEGMENTS)
+					continue;
+
+				Console.WriteLine ("WARNING! Deleting stale data {0}", file);
+				directory.DeleteFile (file);
+			}
+		}
 	}
-}
\ No newline at end of file
+}
