@@ -26,7 +26,7 @@ using BitVector = Lucene.Net.Util.BitVector;
 namespace Lucene.Net.Index
 {
 	
-	/// <version>  $Id: SegmentReader.cs,v 1.7 2006/10/02 17:09:00 joeshaw Exp $
+	/// <version>  $Id: SegmentReader.cs,v 1.8 2006/11/29 19:25:43 joeshaw Exp $
 	/// </version>
 	public class SegmentReader : IndexReader
 	{
@@ -327,6 +327,16 @@ namespace Lucene.Net.Index
 			}
 		}
 		
+		public override Document Document(int n, string[] fields)
+		{
+			lock (this)
+			{
+				if (IsDeleted(n))
+					throw new System.ArgumentException("attempt to access a deleted document");
+				return fieldsReader.Doc(n, fields);
+			}
+		}
+		
 		public override bool IsDeleted(int n)
 		{
 			lock (this)
@@ -520,7 +530,7 @@ namespace Lucene.Net.Index
             byte[] ones = new byte[size];
             byte val = DefaultSimilarity.EncodeNorm(1.0f);
             for (int index = 0; index < size; index++)
-                ones.SetValue(val, index);
+                ones [index] = val;
 
             return ones;
 		}
@@ -725,4 +735,4 @@ namespace Lucene.Net.Index
             }
 		}
 	}
-}
\ No newline at end of file
+}
