@@ -1,7 +1,7 @@
 //
 // LuceneCommon.cs
 //
-// Copyright (C) 2004-2005 Novell, Inc.
+// Copyright (C) 2004-2007 Novell, Inc.
 //
 
 //
@@ -35,6 +35,7 @@ using System.Xml;
 using System.Xml.Serialization;
 
 using Lucene.Net.Analysis;
+using Lucene.Net.Analysis.Snowball;
 using Lucene.Net.Analysis.Standard;
 using Lucene.Net.Documents;
 using Lucene.Net.Index;
@@ -78,7 +79,10 @@ namespace Beagle.Daemon {
 		//     lower case so that we're truly case insensitive.
 		// 16: add inverted timestamp to make querying substantially faster
 		// 17: add boolean property to denote a child indexable
-		private const int MAJOR_VERSION = 17;
+		// 18: add IsPersistent to properties, and adjust coded values
+		//     in AddPropertyToDocument() and GetPropertyFromDocument();
+		//     changed subdate field format rules for better readability
+		private const int MAJOR_VERSION = 18;
 		private int minor_version = 0;
 
 		private string index_name;
@@ -95,6 +99,9 @@ namespace Beagle.Daemon {
 		// expect to have change.  Canonical example: file names.
 		private Lucene.Net.Store.Directory secondary_store = null;
 
+		// Flush if more than this number of requests
+		public const int RequestFlushThreshold = 37; // a total arbitrary magic number
+
 		//////////////////////////////////////////////////////////////////////////////
 
 		protected LuceneCommon (string index_name, int minor_version)
@@ -468,6 +475,7 @@ namespace Beagle.Daemon {
 			private char [] buffer = new char [2];
 			private bool strip_extra_property_info = false;
 			private bool tokenize_email_hostname = false;
+			const string DEFAULT_STEMMER = "English";
 
 			public BeagleAnalyzer (bool is_indexing_analyzer)
 			{
@@ -524,7 +532,7 @@ namespace Beagle.Daemon {
 				    || fieldName == "PropertyText"
 				    || is_text_prop) {
 					outstream = new NoiseEmailHostFilter (outstream, tokenize_email_hostname);
-					outstream = new PorterStemFilter (outstream);
+					outstream = new SnowballFilter (outstream, DEFAULT_STEMMER);
 				}
 
 				return outstream;
@@ -598,13 +606,13 @@ namespace Beagle.Daemon {
 			DateTime dt = StringFu.StringToDateTime (prop.Value);
 
 			Field f;
-			f = new Field ("YM:" + field_name,
+			f = new Field (field_name + "(YM)",
 				       StringFu.DateTimeToYearMonthString (dt),
 				       Field.Store.NO,
 				       Field.Index.NO_NORMS);
 			doc.Add (f);
 
-			f = new Field ("D:" + field_name,
+			f = new Field (field_name + "(D)",
 				       StringFu.DateTimeToDayString (dt),
 				       Field.Store.NO,
 				       Field.Index.NO_NORMS);
@@ -613,7 +621,7 @@ namespace Beagle.Daemon {
 
 		static protected void AddPropertyToDocument (Property prop, Document doc)
 		{
-			if (prop == null || prop.Value == null)
+			if (prop == null || String.IsNullOrEmpty (prop.Value))
 				return;
 
 			// Don't actually put properties in the UnindexedNamespace
@@ -643,8 +651,9 @@ namespace Beagle.Daemon {
 			}
 
 			string coded_value;
-			coded_value = String.Format ("{0}:{1}",
+			coded_value = String.Format ("{0}{1}:{2}",
 						     prop.IsSearched ? 's' : '_',
+						     prop.IsPersistent ? 'p' : '_',
 						     prop.Value);
 
 			string field_name = PropertyToFieldName (prop.Type, prop.Key);
@@ -659,6 +668,8 @@ namespace Beagle.Daemon {
 				AddDateFields (field_name, prop, doc);
 		}
 
+		public static bool DumpIndexMode = false;
+
 		static protected Property GetPropertyFromDocument (Field f, Document doc, bool from_primary_index)
 		{
 			// Note: we don't use the document that we pass in,
@@ -669,23 +680,38 @@ namespace Beagle.Daemon {
 			if (f == null)
 				return null;
 
+			bool internal_prop = false;
+
 			string field_name;
 			field_name = f.Name ();
 			if (field_name.Length < 7
-			    || ! field_name.StartsWith ("prop:"))
-				return null;
+			    || ! field_name.StartsWith ("prop:")) {
+				if (DumpIndexMode)
+					internal_prop = true;
+				else
+					return null;
+			}
 
 			string field_value;
 			field_value = f.StringValue ();
 
 			Property prop;
 			prop = new Property ();
-			prop.Type = CodeToType (field_name [5]);
-			prop.Key = field_name.Substring (7);
-			prop.Value = field_value.Substring (2);
+
+			if (DumpIndexMode) {
+				prop.Type = CodeToType ( internal_prop ? 'k' : field_name [5]);
+				prop.Key = (internal_prop ? field_name : field_name.Substring (7));
+				prop.Value = (internal_prop ? field_value : field_value.Substring (3));
+			} else {
+				prop.Type = CodeToType (field_name [5]);
+				prop.Key = field_name.Substring (7);
+				prop.Value = field_value.Substring (3);
+			}
+
 			prop.IsSearched = (field_value [0] == 's');
+			prop.IsPersistent = (field_value [1] == 'p');
 			prop.IsMutable = ! from_primary_index;
-			prop.IsStored = f.IsStored ();
+			prop.IsStored = true; // Unstored fields cannot be retrieved
 
 			return prop;
 		}
@@ -705,6 +731,11 @@ namespace Beagle.Daemon {
 
 			Field f;
 
+			// During querying, we retrieve a lucene document with only certain fields
+			// like Uri and Timestamp and quickly check if the document is a good one
+			// The field specified document constructor runs faster if the fields that
+			// are asked for are located at the beginning of the document.
+			// Hence it is better to keep "Uri" and "Timestamp" at the beginning.
 			f = new Field ("Uri", UriFu.UriToEscapedString (indexable.Uri),
 				       Field.Store.YES, Field.Index.NO_NORMS);
 			primary_doc.Add (f);
@@ -732,21 +763,22 @@ namespace Beagle.Daemon {
 				// Create an inverted timestamp so that we can
 				// sort by timestamp at search-time.
 				long timeval = Convert.ToInt64 (str);
-				f = new Field ("InvertedTimestamp", (Int64.MaxValue - timeval).ToString (),
+				// Pad the inverted timestamp with zeroes for proper string comparison during termenum enumeration
+				f = new Field ("InvertedTimestamp", (Int64.MaxValue - timeval).ToString ("d19"),
 					       Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 
 				str = StringFu.DateTimeToYearMonthString (indexable.Timestamp);
-				f = new Field ("YM:Timestamp", str, Field.Store.YES, Field.Index.NO_NORMS);
+				f = new Field ("Timestamp(YM)", str, Field.Store.YES, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
-				f = new Field ("YM:" + wildcard_field, str,
+				f = new Field (wildcard_field + "(YM)", str,
 					       Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 
 				str = StringFu.DateTimeToDayString (indexable.Timestamp);
-				f = new Field ("D:Timestamp", str, Field.Store.YES, Field.Index.NO_NORMS);
+				f = new Field ("Timestamp(D)", str, Field.Store.YES, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
-				f = new Field ("D:" + wildcard_field, str,
+				f = new Field (wildcard_field + "(D)", str,
 					       Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 			}
@@ -773,12 +805,13 @@ namespace Beagle.Daemon {
 					primary_doc.Add (f);
 				}
 			
+				// FIXME: HotText is ignored for now!
 				// Then add "HotText"
-				reader = indexable.GetHotTextReader ();
-				if (reader != null) {
-					f = new Field ("HotText", reader);
-					primary_doc.Add (f);
-				}
+				//reader = indexable.GetHotTextReader ();
+				//if (reader != null) {
+				//	f = new Field ("HotText", reader);
+				//	primary_doc.Add (f);
+				//}
 			}
 
 			// Store the Type and MimeType in special properties
@@ -866,6 +899,14 @@ namespace Beagle.Daemon {
 			// return w/o doing anything.
 			foreach (Property prop in prop_only_indexable.Properties) {
 				seen_props [prop.Key] = prop;
+
+				// Don't add properties that are empty; they
+				// essentially mean "reset this property"
+				if (prop.Value == String.Empty) {
+					Log.Debug ("Resetting prop '{0}'", prop.Key);
+					continue;
+				}
+
 				AddPropertyToDocument (prop, new_doc);
 				Logger.Log.Debug ("New prop '{0}' = '{1}'", prop.Key, prop.Value);
 			}
@@ -887,6 +928,27 @@ namespace Beagle.Daemon {
 			return new_doc;
 		}
 
+		static protected Document MergeDocuments (Document doc,
+							  Document prop_change_doc)
+		{
+			if (doc == null)
+				return prop_change_doc;
+
+			if (prop_change_doc == null)
+				return doc;
+
+			foreach (Field f in prop_change_doc.Fields ()) {
+				Property prop = GetPropertyFromDocument (f, prop_change_doc, false);
+
+				if (prop != null && prop.IsPersistent) {
+					Log.Debug ("Moving old persistent prop '{0}' = '{1}' forward", prop.Key, prop.Value);
+					AddPropertyToDocument (prop, doc);
+				}
+			}
+
+			return doc;
+		}
+
 		static protected Uri GetUriFromDocument (Document doc)
 		{
 			string uri;
@@ -912,11 +974,6 @@ namespace Beagle.Daemon {
 
 			AddPropertiesToHit (hit, doc, true);
 
-			// Get the Type and MimeType from the properties.
-			hit.Type = hit.GetFirstProperty ("beagle:HitType");
-			hit.MimeType = hit.GetFirstProperty ("beagle:MimeType");
-			hit.Source = hit.GetFirstProperty ("beagle:Source");
-
 			return hit;
 		}
 
@@ -975,11 +1032,20 @@ namespace Beagle.Daemon {
 		// Access to the stemmer and list of stop words
 		//
 
-		static PorterStemmer stemmer = new PorterStemmer ();
+		static SF.Snowball.Ext.EnglishStemmer stemmer = new SF.Snowball.Ext.EnglishStemmer ();
 
 		static public string Stem (string str)
 		{
-			return stemmer.Stem (str);
+			string stemmed_str;
+
+			lock (stemmer) {
+				stemmer.SetCurrent (str);
+				stemmer.Stem ();
+				stemmed_str = stemmer.GetCurrent ();
+				stemmer.SetCurrent (String.Empty);
+			}
+
+			return stemmed_str;
 		}
 
 		public static bool IsStopWord (string stemmed_word)
@@ -1128,7 +1194,7 @@ namespace Beagle.Daemon {
 
 		static private Term NewYearMonthTerm (string field_name, int y, int m)
 		{
-			return new Term ("YM:" + field_name, String.Format ("{0}{1:00}", y, m));
+			return new Term (field_name + "(YM)", String.Format ("{0}{1:00}", y, m));
 		}
 
 		static private LNS.Query NewYearMonthQuery (string field_name, int y, int m)
@@ -1145,11 +1211,14 @@ namespace Beagle.Daemon {
 
 		static private Term NewDayTerm (string field_name, int d)
 		{
-			return new Term ("D:" + field_name, String.Format ("{0:00}", d));
+			return new Term (field_name + "(D)", String.Format ("{0:00}", d));
 		}
 
 		static private LNS.Query NewDayQuery (string field_name, int d1, int d2)
 		{
+			if (d1 == d2)
+				return new LNS.TermQuery (NewDayTerm (field_name, d1));
+
 			return new LNS.RangeQuery (NewDayTerm (field_name, d1),
 						   NewDayTerm (field_name, d2),
 						   true); // query is inclusive
@@ -1490,6 +1559,23 @@ namespace Beagle.Daemon {
 				return;
 			}
 
+			if (abstract_part is QueryPart_Uri) {
+				QueryPart_Uri part = (QueryPart_Uri) abstract_part;
+
+				// Do a term query on the Uri field.
+				// This is probably less efficient that using a TermEnum;
+				// but this is required for the query API where the uri query
+				// can be part of a prohibited query or a boolean or query.
+				Term term;
+				term = new Term ("Uri", UriFu.UriToEscapedString (part.Uri));
+				if (term_list != null)
+					term_list.Add (term);
+				primary_query = new LNS.TermQuery (term);
+
+				// Query only the primary index
+				return;
+			}
+
 			if (abstract_part is QueryPart_DateRange) {
 
 				QueryPart_DateRange part = (QueryPart_DateRange) abstract_part;
