@@ -1,7 +1,7 @@
 //
 // LuceneQueryable.cs
 //
-// Copyright (C) 2004-2005 Novell, Inc.
+// Copyright (C) 2004-2007 Novell, Inc.
 //
 
 //
@@ -26,6 +26,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 
 using Beagle.Util;
@@ -116,15 +117,15 @@ namespace Beagle.Daemon {
 			Shutdown.ShutdownEvent += new Shutdown.ShutdownHandler (OnShutdownEvent);
 		}
 
-		protected string IndexName {
+		public string IndexName {
 			get { return index_name; }
 		}
 
-		protected string IndexDirectory {
+		public string IndexDirectory {
 			get { return driver.TopDirectory; }
 		}
 
-		protected string IndexFingerprint {
+		public string IndexFingerprint {
 			get { return driver.Fingerprint; }
 		}
 
@@ -170,6 +171,11 @@ namespace Beagle.Daemon {
 			return true;
 		}
 
+		virtual public bool HasUri (Uri uri)
+		{
+			return Driver.HasUri (uri);
+		}
+
 		/////////////////////////////////////////
 
 		virtual protected bool HitIsValid (Uri uri)
@@ -342,7 +348,7 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////
 
-		protected string GetSnippetFromTextCache (string [] query_terms, Uri uri)
+		protected SnippetReader GetSnippetFromTextCache (string [] query_terms, Uri uri, bool full_text)
 		{
 			// Look up the hit in our text cache.  If it is there,
 			// use the cached version to generate a snippet.
@@ -352,71 +358,48 @@ namespace Beagle.Daemon {
 			if (reader == null)
 				return null;
 
-			string snippet = SnippetFu.GetSnippet (query_terms, reader);
-			reader.Close ();
-
-			return snippet;
+			return SnippetFu.GetSnippet (query_terms, reader, full_text);
 		}
 
-		// When remapping, override this with
-		// return GetSnippetFromTextCache (query_terms, remapping_fn (hit.Uri))
-		virtual public string GetSnippet (string [] query_terms, Hit hit)
+		virtual public ISnippetReader GetSnippet (string [] query_terms, Hit hit, bool full_text)
 		{
-			return GetSnippetFromTextCache (query_terms, hit.Uri);
+			return GetSnippetFromTextCache (query_terms, hit.Uri, full_text);
 		}
 
 		/////////////////////////////////////////
 
-		private int progress_percent = -1;
-		private QueryableState state = QueryableState.Idle;
 		private DateTime last_state_change = DateTime.MinValue;
 
 		public QueryableStatus GetQueryableStatus ()
 		{
 			QueryableStatus status = new QueryableStatus ();
 
-			status.State = state;
-			status.ProgressPercent = progress_percent;
-
-			// If we're in read-only mode, query the driver
-			// and not the indexer for the item count.
-			if (indexer == null)
-				status.ItemCount = driver.GetItemCount ();
-			else
-				status.ItemCount = indexer.GetItemCount ();
-
-			// Frequent state changes are common, and there isn't
-			// a real state machine with continuity when it comes
-			// to the indexing process.  A delayed indexing task,
-			// for example, might not actually run for several
-			// seconds after it is scheduled.  In this case, the
-			// backend might be in an "Idle" state, but the
-			// indexing process clearly isn't done.  To work
-			// around this, we also track the last time the state
-			// changed.  If it's less than some threshold, then
-			// we consider ourselves to still be in the process of
-			// indexing.
-			if (state != QueryableState.NotApplicable
-			    && (state != QueryableState.Idle
-				|| (DateTime.Now - last_state_change).TotalSeconds <= 30))
-				status.IsIndexing = true;
+			status.ProgressPercent = this.ProgressPercent;
+			status.ItemCount = driver.GetItemCount ();
+			status.IsIndexing = this.IsIndexing;
 
 			return status;
 		}
 
-		public QueryableState State {
-			get { return this.state; }
-			set { 
-				//Logger.Log.Debug ("State {0}: {1} -> {2}", this, this.state, value);
+		/////////////////////////////////////////
 
-				this.state = value;
-				this.last_state_change = DateTime.Now;
+		private bool is_indexing = false;
+
+		// Reports whether the backend is performing the initial crawling and indexing
+		protected bool IsIndexing {
+			get { return is_indexing; }
+			set {
+				bool changed = (is_indexing != value);
+
+				is_indexing = value;
+
+				if (changed)
+					QueryDriver.QueryableChanged (this, null);
 			}
 		}
 
-		public int ProgressPercent {
-			get { return this.progress_percent; }
-			set { this.progress_percent = value; }
+		protected virtual int ProgressPercent {
+			get { return -1; }
 		}
 
 		/////////////////////////////////////////
@@ -476,8 +459,8 @@ namespace Beagle.Daemon {
 
 		virtual protected bool PreAddIndexableHook (Indexable indexable)
 		{
-			// By default, we like everything.
-			return true;
+			// By default, we like everything if we are not read-only
+			return ! read_only_mode;
 		}
 
 		// If we are remapping Uris, indexables should be added to the
@@ -485,14 +468,17 @@ namespace Beagle.Daemon {
 		// will come back w/ an internal Uri.  In order for change
 		// notification to work correctly, we have to map it to
 		// an external Uri.
-		virtual protected void PostAddHook (Indexable indexable, IndexerAddedReceipt receipt)
+		// Return the remapped uri.
+		virtual protected Uri PostAddHook (Indexable indexable, IndexerAddedReceipt receipt)
 		{
-			// Does nothing by default
+			// By default, remapped uri is the indexable uri
+			return indexable.Uri;
 		}
 
-		virtual protected void PostRemoveHook (Indexable indexable, IndexerRemovedReceipt receipt)
+		virtual protected Uri PostRemoveHook (Indexable indexable, int num_removed)
 		{
-			// Does nothing by default
+			// By default, remapped uri is the indexable uri
+			return indexable.Uri;
 		}
 
 		//////////////////////////////////////////////////////////////////////////////////
@@ -514,19 +500,10 @@ namespace Beagle.Daemon {
 
 			override protected void DoTaskReal ()
 			{
-				QueryableState old_state = queryable.State;
-				queryable.State = QueryableState.Indexing;
-
 				if (queryable.PreAddIndexableHook (indexable)) {
 					queryable.AddIndexable (indexable);
-
-					if (Priority == Scheduler.Priority.Immediate)
-						queryable.Flush ();
-					else
-						queryable.ConditionalFlush ();
+					queryable.ConditionalFlush ();
 				}
-
-				queryable.State = old_state;
 			}
 
 			override protected void DoCleanup ()
@@ -559,19 +536,21 @@ namespace Beagle.Daemon {
 				this.Tag = generator.StatusName;
 			}
 
+			protected override string StatusName {
+				get { return generator.StatusName; }
+			}
+
 			override protected void DoTaskReal ()
 			{
 				// Since this is a generator, we want the task to
 				// get re-scheduled after it is run.
 				Reschedule = true;
 
-				QueryableState old_state = queryable.State;
-				queryable.State = QueryableState.Indexing;
-
 				// Number of times a null indexable was returned.  We don't want
 				// to spin tightly in a loop here if we're not actually indexing
 				// things.
 				int misfires = 0;
+				bool flushed = false;
 
 				do {
 					if (! generator.HasNextIndexable ()) {
@@ -602,11 +581,12 @@ namespace Beagle.Daemon {
 						generated.Cleanup ();
 					
 					// We keep adding indexables until a flush goes through.
-				} while (! queryable.ConditionalFlush ());
+				} while (! (flushed = queryable.ConditionalFlush ()));
+
+				if (! flushed)
+					queryable.Flush ();
 
 				generator.PostFlushHook ();
-
-				queryable.State = old_state;
 			}
 
 			override protected void DoCleanup ()
@@ -805,10 +785,10 @@ namespace Beagle.Daemon {
 		// Other hooks
 
 		// If this returns true, a task will automatically be created to
-		// add the child.
-		virtual protected bool PreChildAddHook (Indexable child)
+		// add the indexable
+		virtual protected bool PreFilterGeneratedAddHook (Indexable indexable)
 		{
-			return true;
+			return ! read_only_mode;
 		}
 
 		virtual protected void PreFlushHook (IndexerRequest flushed_request)
@@ -843,48 +823,44 @@ namespace Beagle.Daemon {
 		// Returns true if we actually did flush, false otherwise.
 		protected bool ConditionalFlush ()
 		{
-			QueryableState old_state = State;
-			State = QueryableState.Flushing;
-
-			try {
-				lock (request_lock) {
-					if (pending_request.Count > 37) { // a total arbitrary magic number
-						Flush ();
-						return true;
-					}
+			lock (request_lock) {
+				if (pending_request.Count > LuceneCommon.RequestFlushThreshold) {
+					Flush ();
+					return true;
 				}
-				return false;
-			} finally {
-				State = old_state;
 			}
+			return false;
 		}
 
 		protected void Flush ()
 		{
-			QueryableState old_state = State;
-			State = QueryableState.Flushing;
-
-			try {
-				DoFlush ();
-			} finally {
-				State = old_state;
-			}
+			Flush (false);
 		}
 
-		private void DoFlush ()
+		protected void Flush (bool continue_only)
 		{
 			IndexerRequest flushed_request;
 
-			lock (request_lock) {
-				if (pending_request.IsEmpty)
-					return;
+			if (continue_only) {
+				// if the request is merely to signal indexhelper to continue indexing,
+				// then sent a fake indexerrequest but then use the previous request to retrieve
+				// deferred indexables
+				flushed_request = new IndexerRequest ();
+				flushed_request.ContinueIndexing = true;
 
-				flushed_request = pending_request;
-				pending_request = new IndexerRequest ();
+				// Do not pass this through PreFlushHook since this is a fake request
+			} else {
+				lock (request_lock) {
+					if (pending_request.IsEmpty)
+						return;
 
-				// We hold the request_lock when calling PreFlushHook, so
-				// that no other requests can come in until it exits.
-				PreFlushHook (flushed_request);
+					flushed_request = pending_request;
+					pending_request = new IndexerRequest ();
+
+					// We hold the request_lock when calling PreFlushHook, so
+					// that no other requests can come in until it exits.
+					PreFlushHook (flushed_request);
+				}
 			}
 
 			IndexerReceipt [] receipts;
@@ -892,8 +868,16 @@ namespace Beagle.Daemon {
 
 			PostFlushHook (flushed_request, receipts);
 
+			if (continue_only) {
+				flushed_request = pending_request;
+			}
+
 			// Silently return if we get a null back.  This is probably
-			// a bad thing to do.
+			// a bad thing to do. If IndexHelper is shutdown because of
+			// memory blowup or is crashed, then null is returned. Silently
+			// returning means ignoring the indexables in the IndexHelper's
+			// queue (which could be more than what was sent in the last request,
+			// since there could be some deferred-indexables too).
 			if (receipts == null)
 				return;
 
@@ -903,6 +887,7 @@ namespace Beagle.Daemon {
 				return;
 
 			// Update the cached count of items in the driver
+			// FIXME: Verify that this still works after all the deferred-indexable fu
 			driver.SetItemCount (indexer.GetItemCount ());
 
 			// Something happened, so schedule an optimize just in case.
@@ -913,6 +898,7 @@ namespace Beagle.Daemon {
 
 			ArrayList added_uris = new ArrayList ();
 			ArrayList removed_uris  = new ArrayList ();
+			bool indexer_indexable_receipt = false;
 
 			for (int i = 0; i < receipts.Length; ++i) {
 
@@ -920,78 +906,88 @@ namespace Beagle.Daemon {
 					
 					IndexerAddedReceipt r;
 					r = (IndexerAddedReceipt) receipts [i];
+					Indexable indexable = flushed_request.RetrieveRequestIndexable (r);
+
+					if (indexable == null) {
+						Log.Debug ("Should not happen! Previously requested indexable with id #{0} has eloped!", r.Id);
+						continue;
+					}
 
 					// Add the Uri to the list for our change data
-					// before doing any post-processing.
+					// *before* doing any post-processing.
 					// This ensures that we have internal uris when
 					// we are remapping.
-					added_uris.Add (r.Uri);
+					added_uris.Add (indexable.Uri);
 					
 					// Call the appropriate hook
+					Uri notification_uri = indexable.Uri;
 					try {
 						// Map from internal->external Uris in the PostAddHook
-						PostAddHook (flushed_request.GetByUri (r.Uri), r);
+						notification_uri = PostAddHook (indexable, r);
 					} catch (Exception ex) {
 						Logger.Log.Warn (ex, "Caught exception in PostAddHook '{0}' '{1}' '{2}'",
-								 r.Uri, r.FilterName, r.FilterVersion);
+								 indexable.Uri, r.FilterName, r.FilterVersion);
 					}
 
 					// Every added Uri also needs to be listed as removed,
 					// to avoid duplicate hits in the query.  Since the
 					// removed Uris need to be external Uris, we add them
 					// to the list *after* post-processing.
-					removed_uris.Add (r.Uri);
-
+					removed_uris.Add (notification_uri);
 
 				} else if (receipts [i] is IndexerRemovedReceipt) {
 
 					IndexerRemovedReceipt r;
 					r = (IndexerRemovedReceipt) receipts [i];
-					
-					// Drop the removed item from the text cache
-					TextCache.UserCache.Delete (r.Uri);
 
-					
+					Indexable indexable = flushed_request.RetrieveRequestIndexable (r);
+					if (indexable == null) { // Should never happen
+						Log.Warn ("Unable to match indexable-remove #{0} to any request!", r.Id);
+						continue;
+					}
+
 					// Call the appropriate hook
+					Uri notification_uri = indexable.Uri;
 					try {
-						PostRemoveHook (flushed_request.GetByUri (r.Uri), r);
+						notification_uri = PostRemoveHook (indexable, r.NumRemoved);
 					} catch (Exception ex) {
 						Logger.Log.Warn (ex, "Caught exception in PostRemoveHook '{0}'",
-								 r.Uri);
+								 indexable.Uri);
 					}
 
+					// If nothing was removed, no need for change notification
+					if (r.NumRemoved <= 0)
+						continue;
+
 					// Add the removed Uri to the list for our
 					// change data.  This will be an external Uri
 					// when we are remapping.
-					removed_uris.Add (r.Uri);
+					removed_uris.Add (notification_uri);
 					
-				} else if (receipts [i] is IndexerChildIndexablesReceipt) {
-					
-					IndexerChildIndexablesReceipt r;
-					r = (IndexerChildIndexablesReceipt) receipts [i];
-
-					foreach (Indexable child in r.Children) {
-						bool please_add_a_new_task = false;
-
-						try {
-							please_add_a_new_task = PreChildAddHook (child);
-						} catch (InvalidOperationException ex) {
-							// Queryable does not support adding children
-						} catch (Exception ex) {
-							Logger.Log.Warn (ex, "Caught exception in PreChildAddHook '{0}'", child.DisplayUri);
-						}
-
-						if (please_add_a_new_task) {
-							//Logger.Log.Debug ("Adding child {0}", child.Uri);
-							Scheduler.Task task = NewAddTask (child);
-							task.SubPriority = 1;
-							ThisScheduler.Add (task);
-						} else
-							child.Cleanup ();
-					}
+				} else if (receipts [i] is IndexerIndexablesReceipt) {
+					indexer_indexable_receipt = true;
 				}
 			}
 
+			if (! continue_only) {
+				lock (request_lock) {
+					pending_request.DeferredIndexables = flushed_request.DeferredIndexables;
+				}
+			}
+
+			if (indexer_indexable_receipt) {
+				Log.Debug ("Indexing of indexer generated indexables is paused. Scheduling job to continue.");
+				// Create a task asking the indexer to continue indexing
+				Scheduler.Task task;
+				task = Scheduler.TaskFromHook (new Scheduler.TaskHook (ContinueIndexerIndexableIndexing));
+				// Schedule it so that it is the immediate next task to be scheduled
+				task.Priority = Scheduler.Priority.Immediate;
+				task.SubPriority = 100;
+				task.Source = this;
+				task.Tag = "Continue indexing generated indexables from " + IndexName;
+				ThisScheduler.Add (task);
+			}
+
 			if (fa_store != null)
 				fa_store.CommitTransaction ();
 
@@ -1006,6 +1002,12 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		// Ouch! What a name ?!
+		private void ContinueIndexerIndexableIndexing (Scheduler.Task task)
+		{
+			Flush (true);
+		}
+
 		//////////////////////////////////////////////////////////////////////////////////
 
 		//
