@@ -36,6 +36,8 @@ using Beagle.Util;
 
 namespace Beagle.Daemon {
 
+	[PropertyKeywordMapping (Keyword="ext", PropertyName="beagle:FilenameExtension", IsKeyword=true, Description="File extension, e.g. ext:jpeg. Use ext: to search in files with no extension.")]
+
 	public class StaticQueryable : LuceneQueryable 	{
 		
 		protected TextCache text_cache;
@@ -53,7 +55,7 @@ namespace Beagle.Daemon {
 			}
 		}
 
-		override public string GetSnippet (string[] query_terms, Hit hit) 
+		override public ISnippetReader GetSnippet (string[] query_terms, Hit hit, bool full_text) 
 		{
 			if (text_cache == null)
 				return null;
@@ -63,10 +65,7 @@ namespace Beagle.Daemon {
 			if (reader == null)
 				return null;
 			
-			string snippet = SnippetFu.GetSnippet (query_terms, reader);
-			reader.Close ();
-			
-			return snippet;
+			return SnippetFu.GetSnippet (query_terms, reader, full_text);
 		}
 
 		override protected bool HitIsValid (Uri uri)
@@ -78,7 +77,8 @@ namespace Beagle.Daemon {
 			// FIXME: This is a hack, we need to support parent Uri's in some sane way
 			try {
 				int j = uri.LocalPath.LastIndexOf ('#');
-				return File.Exists ((j == -1) ? uri.LocalPath : uri.LocalPath.Substring (0, j));
+				string actual_path = ((j == -1) ? uri.LocalPath : uri.LocalPath.Substring (0, j));
+				return File.Exists (actual_path) || Directory.Exists (actual_path);
 			} catch (Exception e) {
 				Logger.Log.Warn ("Exception executing HitIsValid on {0}", uri.LocalPath);
 				return false;
