@@ -1,6 +1,6 @@
 using System;
 using System.Diagnostics;
-using Mono.Posix;
+using Mono.Unix;
 using Gtk;
 
 namespace Search.Pages {
@@ -9,18 +9,20 @@ namespace Search.Pages {
 
 	public class StartDaemon : Base {
 
-		public DaemonStarted DaemonStarted;
-
-		private Gtk.Button button;
+		public event DaemonStarted DaemonStarted;
 
 		public StartDaemon ()
 		{
-			HeaderIconStock = Stock.DialogError;
-			HeaderMarkup = "<big><b>" + Catalog.GetString ("Daemon not running") + "</b></big>";
+			HeaderIconFromStock = Stock.DialogError;
+			Header = Catalog.GetString ("Search service not running");
 
-			button = new Gtk.Button (Catalog.GetString ("Start the daemon"));
+			Append (Catalog.GetString ("The search service does not appear to be running. " +
+						   "You can start it by clicking the button below."));
+
+			Gtk.Button button = new Gtk.Button (Catalog.GetString ("Start search service"));
 			button.Clicked += OnStartDaemon;
 			button.Show ();
+
 			Append (button);
 		}
 
@@ -45,7 +47,9 @@ namespace Search.Pages {
 
 		private bool DaemonStartedTimeout ()
 		{
-			DaemonStarted ();
+			if (DaemonStarted != null)
+				DaemonStarted ();
+
 			return false;
 		}
 	}
