@@ -5,16 +5,22 @@
 //
 
 using System;
+using System.Reflection;
 using System.Collections;
+using System.Diagnostics;
 
 using Gtk;
+using Mono.Unix;
+
 using Beagle;
 using Beagle.Util;
-using Mono.Unix;
 
 using Search.Tiles;
 using Search.Tray;
 
+[assembly: AssemblyTitle ("beagle-search")]
+[assembly: AssemblyDescription ("GUI interface to the Beagle search system")]
+
 namespace Search {
 
 	public class MainWindow : Window {
@@ -50,17 +56,16 @@ namespace Search {
 		private XKeybinder keybinder = new XKeybinder ();
 
 		public static bool IconEnabled = false;
+		private static bool search_docs = false;
 
 		public static void Main (string [] args)
 		{
 			SystemInformation.SetProcessName ("beagle-search");
-
 			Catalog.Init ("beagle", ExternalStringsHack.LocaleDir);
 
 			string query = ParseArgs (args);
 
-			Gnome.Program program = new Gnome.Program ("search", "0.0",
-								   Gnome.Modules.UI, args);
+			Gnome.Program program = new Gnome.Program ("search", "0.0", Gnome.Modules.UI, args);
 
 			MainWindow window = new MainWindow ();
 
@@ -84,12 +89,17 @@ namespace Search {
 					PrintUsageAndExit ();
 					return null;
 
+				case "--version":
+					VersionFu.PrintVersion ();
+					Environment.Exit (0);
+					break;
+
 				case "--icon":
 					IconEnabled = true;
 					break;
 
-				case "--autostarted":
-					// FIXME: This option is deprecated and will be removed in a future release.
+				case "--search-docs":
+					search_docs = true;
 					break;
 
 				// Ignore session management
@@ -118,16 +128,15 @@ namespace Search {
 
 		public static void PrintUsageAndExit ()
 		{
-			string usage =
-				"beagle-search: GUI interface to the Beagle search system.\n" +
-				"Web page: http://www.beagle-project.org/\n" +
-				"Copyright (C) 2005-2006 Novell, Inc.\n\n";
+			VersionFu.PrintHeader ();
 
-			usage +=
+			string usage =
 				"Usage: beagle-search [OPTIONS] [<query string>]\n\n" +
 				"Options:\n" +
+				"  --icon\t\t\tAdd an icon to the notification area rather than opening a search window.\n" +
+				"  --search-docs\t\t\tAlso search the system-wide documentation index.\n" +
 				"  --help\t\t\tPrint this usage message.\n" +
-				"  --icon\t\t\tAdd an icon to the notification area rather than opening a search window.\n";
+				"  --version\t\t\tPrint version information.\n";
 
 			Console.WriteLine (usage);
 			System.Environment.Exit (0);
@@ -136,7 +145,7 @@ namespace Search {
 		public MainWindow () : base (WindowType.Toplevel)
 		{
 			Title = Catalog.GetString ("Desktop Search");
-			Icon = Beagle.Images.GetPixbuf ("system-search.png");
+			Icon = WidgetFu.LoadThemeIcon ("system-search", 16);
 
 			DefaultWidth = 700;
 			DefaultHeight = 550;
@@ -146,6 +155,7 @@ namespace Search {
 			vbox.Spacing = 3;
 
 			uim = new UIManager (this);
+			uim.DomainChanged += OnDomainChanged;
 			uim.ScopeChanged += OnScopeChanged;
 			uim.SortChanged += OnSortChanged;
 			uim.ToggleDetails += OnToggleDetails;
@@ -166,7 +176,7 @@ namespace Search {
 
 			// The auto search after timeout feauture is now optional
 			// and can be disabled.
-			if (Beagle.Util.Conf.Searching.BeagleSearchAutoSearch) {
+			if (Conf.BeagleSearch.GetOption (Conf.Names.BeagleSearchAutoSearch, true)) {
 				entry.Changed += OnEntryChanged;
 				entry.MoveCursor += OnEntryMoveCursor;
 			}
@@ -238,7 +248,8 @@ namespace Search {
 			tips.SetTip (button, Catalog.GetString ("Start searching"), "");
 			tips.Enable ();
 
-			if (Environment.UserName == "root" && ! Conf.Daemon.AllowRoot) {
+			if (Environment.UserName == "root" &&
+			    ! Conf.Daemon.GetOption (Conf.Names.AllowRoot, false)) {
 				pages.CurrentPage = pages.PageNum (rootuser);
 				entry.Sensitive = button.Sensitive = uim.Sensitive = false;
 			} else {
@@ -250,14 +261,19 @@ namespace Search {
 				tray.Clicked += OnTrayActivated;
 				tray.Search += OnTraySearch;
 
-				string binding = Conf.Searching.ShowSearchWindowBinding.ToString ();
+				Config config = Conf.Get (Conf.Names.BeagleSearchConfig);
+				bool binding_ctrl = config.GetOption (Conf.Names.KeyBinding_Ctrl, false);
+				bool binding_alt = config.GetOption (Conf.Names.KeyBinding_Alt, false);
+				string binding_key = config.GetOption (Conf.Names.KeyBinding_Key, "F12");
+
+				string binding = new KeyBinding (binding_key, binding_ctrl, binding_alt).ToString ();
 				string tip_text = Catalog.GetString ("Desktop Search");
 
 				if (binding != String.Empty) {
 					tip_text += String.Format (" ({0})", binding);
 
 					// Attach the hide/show keybinding
-					keybinder.Bind (Conf.Searching.ShowSearchWindowBinding.ToString (), OnTrayActivated);
+					keybinder.Bind (binding, OnTrayActivated);
 				}
 
 				tray.TooltipText = tip_text;
@@ -339,12 +355,14 @@ namespace Search {
 				current_query.AddDomain (QueryDomain.Neighborhood);
 
 				// Don't search documentation by default
-				QueryPart_Property part = new QueryPart_Property ();
-				part.Logic = QueryPartLogic.Prohibited;
-				part.Type = PropertyType.Keyword;
-				part.Key = "beagle:Source";
-				part.Value = "documentation";
-				current_query.AddPart (part);
+				if (! search_docs) {
+					QueryPart_Property part = new QueryPart_Property ();
+					part.Logic = QueryPartLogic.Prohibited;
+					part.Type = PropertyType.Keyword;
+					part.Key = "beagle:Source";
+					part.Value = "documentation";
+					current_query.AddPart (part);
+				}
 
 				current_query.AddText (query);
 				current_query.HitsAddedEvent += OnHitsAdded;
@@ -458,6 +476,21 @@ namespace Search {
 			pages.CurrentPage = pages.PageNum (quicktips);
 		}
 
+		private void OnDomainChanged (QueryDomain domain, bool active)
+		{
+			if (current_query == null)
+				return;
+
+			// FIXME: Most likely refire the query.
+			// Also keep the setting, so it can be used for future queries
+			// in this running instance.
+
+			if (active)
+				current_query.AddDomain (domain);
+			else
+				current_query.RemoveDomain (domain);
+		}
+
 		private void ShowInformation (Tiles.Tile tile)
 		{
 			if (tile != null) {
@@ -485,18 +518,23 @@ namespace Search {
 		{
 			foreach (Hit hit in response.Hits) {
 				Tile tile = TileActivatorOrg.MakeTile (hit, current_query);
-				if (tile == null)
+
+				if (tile == null) {
+					Console.WriteLine ("No tile found for: {0} ({1})", hit.Uri, hit.Type);
 					continue;
+				}
 
 				if (filter != null && !filter.Filter (tile))
 					continue;
 
 				view.AddHit (tile);
+
 				if (pages.CurrentPageWidget != panes)
 					pages.CurrentPage = pages.PageNum (panes);
 			}
 
-			TotalMatches += response.NumMatches;
+			if (response.NumMatches != -1)
+				TotalMatches += response.NumMatches;
 		}
 
 		private void OnHitsSubtracted (HitsSubtractedResponse response)
@@ -509,6 +547,32 @@ namespace Search {
 			CheckNoMatch ();
 		}
 
+#if ENABLE_AVAHI
+                private void OnUnknownHostFound (object sender, AvahiEventArgs args)
+                {
+			NotificationMessage m = new NotificationMessage ();
+			m.Pixbuf = WidgetFu.LoadThemeIcon ("network-workgroup", 48);
+			m.Title = Catalog.GetString ("There are computers near you running Beagle");
+			m.Message = Catalog.GetString ("You can select to search other computers from the \"Search\" menu.");
+			m.AddAction ("Configure", OnNetworkConfigure);
+			notification_area.Display (m);
+		}
+
+		private void OnNetworkConfigure (object o, EventArgs args)
+		{
+			Process p = new Process ();
+			p.StartInfo.UseShellExecute = false;
+			p.StartInfo.FileName = "beagle-settings";
+			p.StartInfo.Arguments = "--networking";
+
+			try {
+				p.Start ();
+			} catch (Exception e) {
+				Console.WriteLine ("Could not start beagle-settings: {0}", e);
+			}
+                }
+#endif
+
 		private void CheckNoMatch ()
 		{
 			MatchType matches = view.MatchState;
