@@ -24,7 +24,6 @@ namespace Search.Tiles {
 		public TileFile (Beagle.Hit hit, Beagle.Query query) : base (hit, query)
 		{
 			Title = GetTitle (hit);
-			EnableOpenWith = true;
 			
 			if (Hit.FileInfo != null) {
 				Timestamp = Hit.FileInfo.LastWriteTimeUtc;
@@ -35,6 +34,12 @@ namespace Search.Tiles {
 			AddAction (new TileAction (Catalog.GetString ("E-Mail"), Email));
 			// AddAction (new TileAction (Catalog.GetString ("Instant-Message"), InstantMessage));
 			AddAction (new TileAction (Catalog.GetString ("Move to Trash"), Gtk.Stock.Delete, MoveToTrash));
+
+			if (! String.IsNullOrEmpty (Hit.GetFirstProperty("dc:author"))) {
+				AddAction(new TileAction ("Find Documents From Same Author", Gtk.Stock.Find, FindSameAuthor));
+			}
+
+			EnableOpenWith = true;
 		}
 
 		static ThumbnailFactory thumbnailer = new ThumbnailFactory ();
@@ -45,6 +50,39 @@ namespace Search.Tiles {
 			// 48 is too small for thumbnails
 			if (!thumbnailer.SetThumbnailIcon (image, Hit, size))
 				base.LoadIcon (image, size);
+
+			// FIXME: Multiple emblems
+			string emblem = Hit.GetFirstProperty ("nautilus:emblem");
+			if (emblem == null)
+				return;
+
+			Gdk.Pixbuf icon_pixbuf = image.Pixbuf.Copy ();
+			Gdk.Pixbuf emblem_pixbuf = WidgetFu.LoadThemeIcon ("emblem-" + emblem, 24);
+
+			if (icon_pixbuf == null || emblem_pixbuf == null) {
+				if (icon_pixbuf != null)
+					icon_pixbuf.Dispose ();
+
+				if (emblem_pixbuf == null)
+					emblem_pixbuf.Dispose ();
+
+				return;
+			}
+
+			// If the icon itself is smaller than our requested
+			// emblem, just display the emblem.
+			if (icon_pixbuf.Height < emblem_pixbuf.Height || icon_pixbuf.Width < emblem_pixbuf.Width) {
+				icon_pixbuf.Dispose ();
+				image.Pixbuf.Dispose ();
+				image.Pixbuf = emblem_pixbuf;
+				return;
+			}
+
+			emblem_pixbuf.Composite (icon_pixbuf, 0, 0,
+						 emblem_pixbuf.Width, emblem_pixbuf.Height,
+						 0, 0, 1, 1, Gdk.InterpType.Bilinear, 255);
+			image.Pixbuf.Dispose ();
+			image.Pixbuf = icon_pixbuf;
 		}
 
 		protected static string GetTitle (Beagle.Hit hit, bool get_parent)
@@ -148,5 +186,20 @@ namespace Search.Tiles {
 
 			return details;
 		}
+		
+		public void FindSameAuthor()
+		{
+			SafeProcess p = new SafeProcess ();
+			string author = Hit.GetFirstProperty("dc:author");
+			if( String.IsNullOrEmpty(author))
+				 author = Hit.GetFirstProperty("dc:creator");
+			p.Arguments = new string [] { "beagle-search", String.Format ("author:{0} OR creator:{0}", author) };
+			try {
+				p.Start () ;
+			} catch (Exception e) {
+				Console.WriteLine ("Error launching new search: " + e.Message);
+			}
+			
+		}
 	}
 }
