@@ -43,36 +43,16 @@ namespace Search.Tiles {
 				Title = Catalog.GetString (String.Format ("Attachment to \"{0}\"", Hit ["parent:dc:title"]));
 
 			Description = Catalog.GetString ("Mail attachment");
-		}
-
-		// FIXME: This needs better handling in the daemon, attachments
-		// architecture sucks
-		private static bool IsAttachment (Beagle.Hit hit)
-		{
-			// check if there is parent and parent has attachments
-			string str = hit ["parent:fixme:hasAttachments"];
-			return (hit.ParentUri != null && str != null && (str == "true"));
-		}
-		
-		private static string GetHitProperty (Beagle.Hit hit, string name)
-		{
-			// FIXME: We should handle this case better, but
-			// for now, if we match an attachment, we just want
-			// to display the properties for the parent message.
-			if (!IsAttachment (hit))
-				return hit [name];
-			else
-				return hit ["parent:" + name];
+			
+			if ((Utils.GetFirstPropertyOfParent (hit, "fixme:isSent") == null)){
+				AddAction ( new TileAction (Catalog.GetString ("Find Messages From Sender"), Gtk.Stock.Find, FindAllFromSender));
+			}
 		}
 
 		public override void Open ()
 		{
-			SafeProcess p;
+			SafeProcess p = MailMessage.GetClientProcess (Hit);
 			
-			if (Hit.ParentUri != null)
-				p = MailMessage.GetClientProcess (GetHitProperty (Hit, "fixme:client"), Hit.EscapedParentUri);
-			else
-				p = MailMessage.GetClientProcess (GetHitProperty (Hit, "fixme:client"), Hit.EscapedUri);
 			if (p == null) {
 				OpenFromMime (Hit);
 				return;
@@ -85,5 +65,17 @@ namespace Search.Tiles {
 			}
 		}
 		
+		public void FindAllFromSender()
+		{
+			SafeProcess p = new SafeProcess ();
+			string addr = Search.Tiles.Utils.GetFirstPropertyOfParent(Hit,"fixme:from_address");
+			p.Arguments = new string [] { "beagle-search", String.Format ("mailfromaddr:{0}", addr) };
+			try {
+				p.Start () ;
+			} catch (Exception e) {
+				Console.WriteLine ("Error launching new search: " + e.Message);
+			}
+		}
+		
 	}
 }
