@@ -19,6 +19,7 @@ namespace Search.Tray
 	public class TrayIcon
 	{
 		private EventBox eventbox;
+		private Tooltips tips;
 		private NotificationArea notification_area;
 		private Menu popup;
 		private ArrayList recent_searches;
@@ -34,8 +35,8 @@ namespace Search.Tray
 			eventbox = new EventBox ();
 			eventbox.ButtonPressEvent += OnClick;
 
-			Gdk.Pixbuf pixbuf = Beagle.Images.GetPixbuf ("system-search.png");
-			eventbox.Add (new Gtk.Image (pixbuf.ScaleSimple (24, 24, Gdk.InterpType.Hyper)));
+			Gdk.Pixbuf pixbuf = WidgetFu.LoadThemeIcon ("system-search", 24);
+			eventbox.Add (new Gtk.Image (pixbuf));
 			
 			notification_area.Add (eventbox);
 			notification_area.ShowAll ();
@@ -45,6 +46,17 @@ namespace Search.Tray
 			popup = MakeMenu (eventbox);
 		}
 
+		public string TooltipText {
+			set {
+				if (tips == null) {
+					tips = new Gtk.Tooltips ();
+					tips.Enable ();
+				}
+
+				tips.SetTip (eventbox, value, null);
+			}
+		}
+
 		private Gtk.Menu MakeMenu (Gtk.Widget parent) 
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
@@ -67,7 +79,11 @@ namespace Search.Tray
 				menu_to_query_map = new Hashtable ();
 
 				foreach (string s in recent_searches) {
-					item = new Gtk.ImageMenuItem (s);
+					// Replace all occurences of '_' with "__"
+					// so that underscores don't become mnemonics.
+					string query = s.Replace ("_", "__");
+
+					item = new Gtk.ImageMenuItem (query);
 					item.Activated += new EventHandler (OnSearch);
 					menu.Append (item);
 					menu_to_query_map [item] = s;
